package ru.ldeloff.servermonitorbot.commands;

import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.telegram.telegrambots.meta.api.methods.send.SendMessage;
import org.telegram.telegrambots.meta.api.objects.Update;
import ru.ldeloff.servermonitorbot.model.user.Role;
import ru.ldeloff.servermonitorbot.model.server.SshServer;
import ru.ldeloff.servermonitorbot.service.MessagingService;
import ru.ldeloff.servermonitorbot.util.UpdateUtil;
import ru.ldeloff.servermonitorbot.model.user.User;
import ru.ldeloff.servermonitorbot.service.SshService;
import ru.ldeloff.servermonitorbot.view.ServerListButtons;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;

@Slf4j
@RequiredArgsConstructor
public abstract class CommandTemplate {

    private final UpdateUtil updateUtil;
    private final ServerListButtons serverListButtons;
    public final SshService sshService;
    private final MessagingService messagingService;
    @Getter
    @Setter
    private List<Role> expectedRole = List.of(Role.ADMIN);
    @Getter
    @Setter
    private String name;

    public boolean execute(Update update) {
        if (!isExecute(update)) {
            return false;
        }

        User user = updateUtil.getUser(update);
        String message = updateUtil.getMessage(update);
        SendMessage answer = new SendMessage();
        answer.setChatId(user.getTelegramId());
        answer.setText(message);

        if (!updateUtil.roleChecker(update, expectedRole)) {
            logNotAuth(user, message);
            messagingService.send(actionForNotAuth(user, answer));
            return true;
        }

        String type = checkTypeCommand(update);
        switch (type) {
            case "aggregate":
                logSuccess(user, message);
                messagingService.send(executeAggregate(user, answer));
                return true;
            case "command":
                logSuccess(user, message);
                messagingService.send(executeCommand(user, answer));
                return true;
            default:
                return false;
        }
    }

    // Методы для проверки команды
    private boolean isExecute(Update update) {
        if (update.hasMessage()) {
            if (update.getMessage().hasText()) {
                String messageText = update.getMessage().getText();
                if (Objects.equals(messageText, name)) {
                    log.debug("Команда {} будет выполнена", messageText);
                    return true;
                }
            }
        } else if (update.hasCallbackQuery()) {
            String [] tags = update.getCallbackQuery().getData().split(":");
            if (tags.length > 1) {
                if (Objects.equals(tags[0], name)) {
                    log.debug("Команда {} будет выполнена", tags[0]);
                    return true;
                }
            } else {
                log.debug("Команда не будет выполнена");
                return false;
            }
        }
        log.debug("Команда не будет выполнена");
        return false;
    }
    private String checkTypeCommand(Update update) {
        if (update.hasMessage()) {
            if (update.getMessage().hasText()) {
                return "aggregate";
            }
        } else if (update.hasCallbackQuery()) {
            String [] tags = update.getCallbackQuery().getData().split(":");
            if (tags.length > 1) {
                return "command";
            } else {
                return "error";
            }
        }
        return "error";
    }


    // Методы при верных условиях
    void logSuccess(User user, String message) {
        log.info("Получена команда '{}' от {} ({}). OK", message, user.getLogin(), user.getTelegramId());
    }

    public SendMessage executeAggregate(User user, SendMessage message) {
        return serverListButtons.uiForm(message);
    }

    SendMessage executeCommand(User user, SendMessage message) {
        String [] tags = Arrays.stream(message.getText().split(":"))
                .map(String::trim)
                .toArray(String[]::new);

        if (tags[1].equals("all")) {
            message.setText(executeCommandAllHost());
        } else {
            message.setText(Objects.requireNonNull(executeCommandSpecificHost(tags[1])));
        }
        return message;
    }
    private String executeCommandAllHost() {
        List<SshServer> servers = sshService.getHealthSshServers();
        StringBuilder response = new StringBuilder();
        servers.forEach(server -> response.append(executeCommandSpecificHost(server.getName())).append("\n"));
        return response.toString();
    }
    public abstract String executeCommandSpecificHost(String serverName);

    // Методы при неверных условиях
    void logNotAuth(User user, String message) {
        log.warn("Получена команда '" + message + "' от " + user.getLogin()
                + " (" + user.getTelegramId() + "). Нет прав");
    }
    private SendMessage actionForNotAuth(User user, SendMessage message) {
        message.setText("У пользователя " + user.getLogin() + " (" + user.getTelegramId()
                + ") недостаточно прав для выполнения этой команды");
        return message;
    }
}
