package ru.ldeloff.servermonitorbot.service;

import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.telegram.telegrambots.meta.api.methods.send.SendMessage;
import ru.ldeloff.servermonitorbot.model.server.ServerStatus;
import ru.ldeloff.servermonitorbot.model.server.SshServers;
import ru.ldeloff.servermonitorbot.model.SshCommand;
import ru.ldeloff.servermonitorbot.model.server.SshServer;
import ru.ldeloff.servermonitorbot.repository.SshRepository;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Service
@Slf4j
@RequiredArgsConstructor
public class SshServiceImpl implements SshService {

    private final SshServers sshServers;
    private final SshRepository sshRepository;
    private final MessagingService messagingService;

    @Override
    public void updateConnection() {
        sshServers.getServers().forEach(sshServer -> {
            try {
                sshRepository.connect(sshServer);
                updateServerStatus(sshServer, ServerStatus.ONLINE);
                log.info("Успешно подключён к {}", sshServer.getHost());
            } catch (JSchException e) {
                if (sshServer.getStatus() == null) {
                    updateServerStatus(sshServer, ServerStatus.OFFLINE);
                }
                if (sshServer.getStatus() == ServerStatus.ONLINE) {
                    updateServerStatus(sshServer, ServerStatus.CONNECTION_LOST);
                }
                log.warn("Не удалось соединиться с {}: {}", sshServer.getHost(), e.getMessage());
            }
        });
    }

    @Override
    public void disconnectAllConnections() {
        sshServers.getServers().forEach(sshRepository::disconnect);
    }

    @Override
    public SendMessage getStatusSessions(SendMessage message) {
        StringBuilder text = new StringBuilder("Статус соединения (может выполняться долго): \n");
        sshServers.getServers().forEach(server -> text.append(server.getName())
                .append(": ")
                .append(checkStatusServer(server))
                .append("\n"));
        message.setText(text.toString());
        return message;
    }

    private String checkStatusServer(SshServer server) {
        if (Objects.isNull(server.getSession())) {
            return "нет соединения";
        }

        if (server.getSession().isConnected()) {
            return "OK";
        } else {
                return "потеряно соединение";
        }

    }

    private boolean checkHealth(SshServer server) {
        if (Objects.isNull(server.getSession())) {
            return false;
        }

        return server.getSession().isConnected();
    }

    @Override
    public List<SshServer> getAllSshServers() {
        return sshServers.getServers();
    }

    @Override
    public List<SshServer> getHealthSshServers() {
        return sshServers.getServers().stream().filter(this::checkHealth).toList();
    }

    @Override
    public SshCommand execute(SshCommand sshCommand) {
        List<SshCommand> sshCommands = new ArrayList<>();
        sshCommands.add(sshCommand);
        return execute(sshCommands).get(0);
    }

    @Override
    public List<SshCommand> execute(List<SshCommand> sshCommands) {
        Session session = sshCommands.get(0).getSshServer().getSession(); // подразумевается, что в листе с командами сервер один и тот же
        ChannelExec channel = null;
        try {
            channel = (ChannelExec) session.openChannel("exec");
            ChannelExec finalChannel = channel;

            sshCommands.forEach(command -> {
                try {
                    finalChannel.setCommand(command.getCommand());
                    ByteArrayOutputStream responseStream = new ByteArrayOutputStream();
                    finalChannel.setOutputStream(responseStream);
                    finalChannel.connect();
                    while (finalChannel.isConnected()) {
                        Thread.sleep(command.getTimeout());
                    }
                    String responseString = new String(responseStream.toByteArray());
                    command.setResponse(responseString);
                } catch (JSchException | InterruptedException e) {
                    log.warn(e.getMessage());
                    throw new RuntimeException(e);
                }
            });
        } catch (JSchException e) {
            log.warn(e.getMessage());
            throw new RuntimeException(e);
        } finally {
            if (channel != null) {
                channel.disconnect();
            }
        }
        return sshCommands;
    }

    public void updateServerStatus(SshServer server, ServerStatus newStatus) {
        ServerStatus oldStatus = server.getStatus();
        if (oldStatus != newStatus) {
            server.setStatus(newStatus);
            messagingService.sendMessageToAdmins("Server status changed for " + server.getName() +
                    ": " + oldStatus + " -> " + newStatus);
        }
    }
}
